'********************************************************
'  Test program for TS-2100, 2200, 2800     Nov. 10, 1999
'********************************************************
'       Program history
'who  when     ver        what
'--- ------    ---    ----------------------------------------
'RLM 11-10-99  2.01  - added jumpers in instructions for TS-2800
'RLM 11-10-99  2.01  - RTS enabled while sending loop back data
'                      (allows 2200 COM1 232 test to work with JP7 present)
'RLM 11-10-99  2.01  - Warning that this program requires special cables
'RLM 12-21-99  2.02  - Moved SRAM address (to D8000h) for TS-2800
'RLM 01-12-00  2.03  - Fixed bug on battery-backed SRAM for TS-2800
'RLM 01-12-00  2.03  - Added support for 128K SRAM for TS-2200
'LAY 06-29-00  2.031 - Added expert menu ; added sub Continue

CONST VERSION = "2.031"      ' Diagnostic Program Version
'********************************************************


DECLARE SUB Instruct485 ()
DECLARE SUB CheckCom1Loop ()
DECLARE SUB FD485 ()
DECLARE SUB StartupScreen ()
DECLARE SUB CharDecode ()
DECLARE SUB Menu ()
DECLARE SUB Instructions ()
DECLARE SUB EtherInfo ()
DECLARE SUB EthernetTest ()
DECLARE SUB EthNumber ()
DECLARE SUB DIO2Test ()
DECLARE SUB LCDtest ()
DECLARE SUB DIO1Test ()
DECLARE SUB ADCtest ()
DECLARE SUB Options ()
DECLARE SUB RunStandard ()
DECLARE SUB EchoCom1 ()
DECLARE SUB SocketTest ()
DECLARE SUB BatteryRAM ()
DECLARE SUB RTCtest ()
DECLARE SUB SendSpace25 ()
DECLARE SUB Bell ()
DECLARE SUB ComXTest (Comnum!, comadr!)
DECLARE SUB SendChar (comadr!, TxD!)
DECLARE SUB GetChar (rx)
DECLARE SUB Continue ()
DECLARE SUB ExprtMenuCharDecode (ExitFlag$)
DECLARE SUB ExprtMenu ()


'********************************************************

CONST p1out = &HF862        ' LED control register
CONST p3ltc = &HF872        ' Par port Strobe

CONST P1PIN = &HF860        ' Port 1 (Push switch register)
CONST P2PIN = &HF868        ' Port 2 (Read only status)
CONST P3PIN = &HF870        ' Port 3 (Read only status)

CONST rtcadd = &H70        ' Real-Time clock address register
CONST rtcdata = &H71       ' Real-Time clock data register

CONST lcdcom = &H72        ' LCD Command register
CONST lcddat = &H73        ' LCD Data register

CONST sramsiz = &H70       ' bit 0 set to "1" for 2200 = 128K SRAM
CONST id1adr = &H74        ' bit 0 set to "1" for 2200 or 2800
CONST sramadr = &H75       ' bit 0 set to "1" if SRAM option present
CONST rs485adr = &H76      ' bit 0 set to "1" if RS-485 option present
CONST srampage = &H77      ' bit 0 = SRAM Page reg. (TS-2200 only)

CONST adcadr = &H7D        ' bit 0 set to "1" if ADC option present
CONST id2adr = &H7C        ' bit 0 set to "1" for 2200, "0" for 2800

CONST adclsb = &H78        ' address of ADC lsb
CONST adcmsb = &H79        ' address of ADC msb
CONST adcstat = &H7E       ' bit 0 set to "1" when ADC conversion done

CONST tmrcon = &H43         ' TIMER Control reg.
CONST tmr2 = &H42           ' timer 2 counter

CONST com2msr = &H2FE       ' COM2 modem status register
CONST com2mcr = &H2FC       ' COM2 modem control register
CONST com2lsr = &H2FD       ' com2 line status
'                          bit 5 = 1 transmit reg. empty (ready for char)
'                          bit 0 = 1 receive char available
CONST com2data = &H2F8      ' COM2 write transmit or read receive data reg.

CONST com1msr = &H3FE       ' COM1 modem status register
CONST com1mcr = &H3FC       ' COM1 modem control register
CONST com1lsr = &H3FD       ' com1 line status
'                          bit 5 = 1 transmit reg. empty (ready for char)
'                          bit 0 = 1 receive char available
CONST com1data = &H3F8      ' COM1 write transmit or read receive data reg.

' Serial offsets - good for any serial port..
CONST TBR = &H0                 ' transmit buffer reg.
CONST RBR = &H0                 ' receive buffer reg.
CONST MCR = &H4                 ' modem control register
CONST LSR = &H5                 ' line status
				'  bit 5 = 1 transmit reg. empty (ready for char)
				'  bit 0 = 1 receive char available
CONST MSR = &H6                 ' modem status register




'********************************************************
' Global Variables
'********************************************************
TYPE regtype         ' define register data type
	ax AS INTEGER
	bx AS INTEGER
	cx AS INTEGER
	dx AS INTEGER
	BP AS INTEGER
	SI AS INTEGER
	DI AS INTEGER
	FLAGS AS INTEGER
END TYPE

COMMON SHARED com1adr, com2adr, Com3Adr, Com4Adr
COMMON SHARED Model, rx, errors, lcdflag, sram
COMMON SHARED inputregs AS regtype, outputregs AS regtype


'********************************************************

begin:

OPEN "CONS:" FOR OUTPUT AS #3
lcdflag = 0     ' only allow initialization once

' initialize Timer 2 for automatic 485 Enable (assume 9600,N,8,1)

OUT tmrcon, &HB2        ' put timer 2 in mode 1
OUT tmr2, &H7E          ' LSbyte for 9600 baud   count = 1918
OUT tmr2, &H7           ' MSbyte for 9600 baud

OUT com2mcr, &H3        ' (COM2) set DTR and RTS asserted true

'********************************************************

DEF SEG = &H40
com1adr = PEEK(&H1) * &H100 + PEEK(&H0)         ' get COM1 serial port address
com2adr = PEEK(&H3) * &H100 + PEEK(&H2)         ' get COM2 serial port address
Com3Adr = PEEK(&H5) * &H100 + PEEK(&H4)         ' check for a serial port card
Com4Adr = PEEK(&H7) * &H100 + PEEK(&H6)

' print Diag program Version #, and BIOS Version, etc.
CALL StartupScreen

WHILE 1
	CALL Menu
	CALL CharDecode
WEND

END

SUB ADCtest
'********************************************************
' Test A/D Converter
'********************************************************

PRINT #3, : PRINT #3,

IF Model <> 28 OR (INP(&H7D) AND 1) = 0 THEN
	CALL Bell
	PRINT #3, " A to D converter not present !!!!!!"
	PRINT #3, "   Wrong model  or  Wrong MACH"
	EXIT SUB
END IF

FOR i = 0 TO 7
	inputregs.bx = 16 + i                       ' 10 Volt scale + chan.#
	inputregs.ax = &HB050                       ' A/D sample
	CALL interrupt(&H15, inputregs, outputregs) ' BIOS Interrupt

	IF INT(outputregs.ax / 256) > 1 THEN         ' AH has error code
		CALL Bell
		PRINT #3, "Error in A/D -- Check hardware";
		EXIT SUB
	END IF
	x = INT(outputregs.bx / 41)      ' convert 0-4096 to %
	PRINT #3, "  Channel"; i; "= "; x; "% of 10V"
NEXT i
PRINT #3,

CALL Continue

END SUB

SUB BatteryRAM

'********************************************************
' Test Battery-backed SRAM
'   same test as "K" except don't write to SRAM first
'********************************************************

IF Model = 28 THEN
	DEF SEG = &HD000           ' 32 Kbyte window to SRAM
ELSEIF sram = 128 THEN
	DEF SEG = &HC000           ' set to 64K window
	OUT srampage, 0            ' set to page 0
ELSE
	DEF SEG = &HC800           ' 32 Kbyte window to SRAM
END IF

PRINT #3, : CALL SendSpace25

IF (INP(sramadr) AND 1) = 0 THEN     ' check for SRAM option
	CALL Bell
	PRINT #3, "SRAM option not enabled in MACH"
	EXIT SUB
END IF

' check for Disk-on-Chip
' and Abort if present
IF PEEK(24576) = &H55 AND PEEK(24577) = &HAA THEN
	CALL Bell
	PRINT #3, "Battery-backed SRAM Test ABORTED  --  DOC installed"
	EXIT SUB
END IF

z = 0
y = 0
FOR i = 1 TO 17000 STEP 15
z = (z + 37) MOD 256
IF PEEK(i) <> z THEN y = y + 1
NEXT i

IF y = 0 THEN
	PRINT #3, "32K Battery-Backed SRAM test passed"
ELSEIF y <> 0 THEN
	CALL Bell
	PRINT #3, "FAILED !!!  32K Battery-Backed SRAM test"
END IF

IF sram = 128 THEN
	OUT srampage, 1         ' set to Page 1
	CALL SendSpace25
	z = 0
	FOR i = 1 TO 17000 STEP 15
		z = (z + 31) MOD 256
		IF PEEK(i) <> z THEN y = y + 1
	NEXT i
	IF y = 0 THEN
		PRINT #3, "128K Battery-Backed SRAM test passed"
	ELSEIF y <> 0 THEN
		CALL Bell
		PRINT #3, "  FAILED !!!  128K Battery-Backed SRAM test"
	END IF
END IF

END SUB

SUB Bell

'************************************************************
' send double bell
'     and a delay after each Bell to pause screen after any failure
'************************************************************

PRINT #3, CHR$(7);
FOR i = 1 TO 2000: NEXT i     ' small delay

PRINT #3, CHR$(7);
FOR i = 1 TO 2000: NEXT i     ' small delay

END SUB

SUB CharDecode
'*******************************************************************
'    get char and decode command
'*******************************************************************

rx = 0
WHILE rx < 32               ' ignore control char
	CALL GetChar(rx)
WEND

PRINT #3, STRING$(1, rx)

SELECT CASE rx
	CASE 69, 101          '  E or e
		FOR i = 1 TO 8: PRINT #3, : NEXT i
		CALL EthernetTest
		CALL Continue
	CASE 66, 98           '  B or b
		CALL BatteryRAM
	CASE 82, 114          '  R or r
		CALL RunStandard
	CASE 83, 115                 ' S or s
		ExitFlag$ = "false"
		DO
			CALL ExprtMenu
			CALL ExprtMenuCharDecode(ExitFlag$)

		LOOP UNTIL LCASE$(ExitFlag$) = "true"
		 

	CASE 49                      ' number 1
		CALL ComXTest(1, com1adr)
	CASE 50                      ' number 2
		CALL ComXTest(2, com2adr)
	CASE 51                      ' number 3
		CALL ComXTest(3, Com3Adr)
	CASE 52                      ' number 4
		CALL ComXTest(4, Com4Adr)

	CASE 72, 104          '  H or h
		CALL Instruct485
		CALL EchoCom1
		CALL Continue
	CASE 70, 102          '  F or f
		CALL Instruct485
		CALL FD485
		CALL Continue
	CASE 65, 97           '  A or a
		CALL ADCtest
	CASE 78, 110          '  N or n
		CALL EthNumber
	CASE 73, 105          '  I or i
		CALL Instructions
	CASE 79, 111          '  O or o
		CALL Options
	CASE 88, 120          '  X or x
		END
	CASE 32               '  Space Bar
		CALL SendSpace25
	CASE ELSE
		CALL Bell
END SELECT

END SUB

SUB CheckCom1Loop
'********************************************************************
' Check for COM1 RS-232 Loop Back cable
'********************************************************************

errors = 0                             ' error count
FOR i = 1 TO 10
	OUT (com1adr + MCR), 2         ' set only RTS asserted (true)
	FOR k = 1 TO 3: NEXT k         ' small delay
	IF (INP(com1adr + MSR) AND 16) = 0 THEN errors = errors + 1  ' get CTS
	OUT (com1adr + MCR), 0         ' set RTS deasserted (false)
	FOR k = 1 TO 3: NEXT k         ' small delay
	IF (INP(com1adr + MSR) AND 16) = 16 THEN errors = errors + 1 ' get CTS
NEXT i
IF errors = 0 THEN
	CALL Bell
	PRINT #3, "  Do NOT have COM1 RS-232 Loop Back cable attached"
	PRINT #3, "     when doing this test"
	PRINT #3, : CALL SendSpace25
	PRINT #3, "This Test has been ABORTED  !!!"
END IF

END SUB

SUB ComXTest (Comnum, comadr)
'********************************************************
' ComXTest - checks the lines on a serial port (needs a loopback plug)
'********************************************************
'  This routine requires a loopback plug with RTS -> CTS and TxD -> RxD
'  It skips data loop back check on COM2

IF comadr = 0 THEN                      ' A simple sanity check
	CALL Bell
	PRINT #3, "COM" + STR$(Comnum) + " Not Present. Returning to Main Menu."
	EXIT SUB
END IF
       
IF Comnum = 2 THEN GOTO handshakeonly

OUT (comadr + MCR), &H2        '  RTS asserted (RS-485 RXD is tri-stated)
			       '  allows JP7 to be installed during 232 test
				 
FOR i = 1 TO 10            ' Flush COM Rx data
	IF (INP((comadr + LSR)) AND 1) = 1 THEN i = INP((comadr + RBR))
NEXT i

CALL SendChar(comadr, 92)
CALL SendChar(comadr, 45)

t = 0: z = 0
FOR i = 1 TO 50
	IF (INP(comadr + LSR) AND 1) THEN
		t = INP(comadr + RBR)
		EXIT FOR
	END IF
NEXT i                                  ' got 1st char
FOR i = 1 TO 50
	x = INP(comadr + LSR) AND 1
	IF x = 1 THEN
		z = INP(comadr + RBR)
		EXIT FOR
	END IF
NEXT i                     ' got 2nd char

CALL SendSpace25

IF t = 92 AND z = 45 THEN
	PRINT #3, "COM" + STR$(Comnum) + " Data Loop Back test passed"
ELSE
	CALL Bell
	PRINT #3, "  FAILED !!!  COM" + STR$(Comnum) + " Data Loop back test"
END IF

'********************************   Check handshake RTS and CTS

handshakeonly:

errors = 0                      ' error count
FOR i = 1 TO 10
	OUT (comadr + MCR), 2   ' set only RTS asserted (true)
	FOR k = 1 TO 3: NEXT k              ' small delay
	IF (INP(comadr + MSR) AND 16) = 0 THEN errors = errors + 1  ' get CTS
	OUT (comadr + MCR), 0   ' set RTS deasserted (false)
	FOR k = 1 TO 3: NEXT k         ' small delay
	IF (INP(comadr + MSR) AND 16) = 16 THEN errors = errors + 1 ' get CTS
NEXT i

CALL SendSpace25
IF errors = 0 THEN
	PRINT #3, "COM" + STR$(Comnum) + " Handshake Loop Back test passed"
ELSE
	CALL Bell
	PRINT #3, "  FAILED !!!  COM" + STR$(Comnum) + " Handshake Loop back test"
END IF

END SUB

SUB Continue

PRINT #3, CHR$(13)
PRINT #3, "Press Any Key to Continue";
PRINT #3, CHR$(13)
CALL GetChar(rx)

END SUB

SUB DIO1Test
'********************************************************
' Test DIO1 Port     (Loop Back Test)
'
'    OUT     IN
'
'   P3.2 -- P3.3
'   P3.0 -- P3.1
'   P3.5 -- P1.0
'   P3.6 -- P1.5
'   DTR2 -- RI2
'   RTS2 -- DSR2
'
' Loop Back Cable must connect the above lines
'
'********************************************************

' Initialize Port1 and Port3 I/O pins *****************************

x = (INP(&HF824)) AND &H90   ' force bits 0-3,5,6 to "0"
OUT &HF824, x                ' set Port3 bits 0-3,5,6, as I/O pins
			     ' bit 4 is IRQ6,   bit 7 is Baud Clock

OUT &HF874, &H9A             ' set Port3 bits 0,2,5,6 as OUTPUTs
			     '   and Port3 bits 1,3 as INPUTs

OUT &HF872, &HFF             ' set Port3 bits 0,2,5,6 high (OUTPUTs)
			     '   and Port3 set bits 1,3 as INPUTs
			     '     (INPUTs must always be kept at "1")

x = (INP(&HF820)) AND &HDE   ' force bit 0 and 5 to "0"
OUT &HF820, x                ' set Port1 bits 0 and 5 to I/O pins

x = (INP(&HF864)) OR &H21    ' force bit 0 and 5 to "1"
OUT &HF864, x                ' set Port1 bits 0 and 5 to INPUTs

x = (INP(&HF862)) OR &H21    ' force bit 0 and 5 to "1"
OUT &HF862, x                ' set Port1 bits 0 and 5 to INPUTs

' **** Start Tests of I/O pins  ***************************************
' w = Port1.5,  x = Port1.0,  y = Port3.1,  z = Port3.3

errors = 0                      ' error count
GOSUB readinputs
IF w + x + y + z <> 4 THEN errors = errors + 1

t = INP(&HF872) AND &HFE     ' set bit 0 to "0"
OUT &HF872, t                ' set Port3 bit 0 = "0" (2,5,6 ="1")
GOSUB readinputs
IF w + x + y + z <> 3 OR y = 1 THEN errors = errors + 1

t = INP(&HF872) AND &HFB     ' set bit 2 to "0"
OUT &HF872, t                ' set Port3 bits 0,2 = "0" (5,6 ="1")
GOSUB readinputs
IF w + x + y + z <> 2 OR z = 1 THEN errors = errors + 1

t = INP(&HF872) AND &HDF     ' set bit 5 to "0"
t = t OR 1                   ' set bit 0 back to "1"
OUT &HF872, t                ' set Port3 bits 2,5 = "0" (0,6 ="1")
GOSUB readinputs
IF w + x + y + z <> 2 OR x = 1 THEN errors = errors + 1

t = INP(&HF872) AND &HBF     ' set bit 6 to "0"
OUT &HF872, t                ' set Port3 bits 2,5,6 = "0" (bit0 ="1")
GOSUB readinputs
IF w + x + y + z <> 1 OR w = 1 THEN errors = errors + 1

CALL SendSpace25
IF errors = 0 THEN
	PRINT #3, "DIO1 Port3 and Port1 Lines test passed"
ELSE
	CALL Bell
	PRINT #3, "  FAILED !!!  DIO1 Port1 and Port3 Lines test"
END IF


'Test DIO1 Port DTR2-RI2  and  RTS2-DSR2 loop back   *******************

comadr = com2adr
errors = 0                      ' error count
FOR i = 1 TO 10
	OUT (comadr + MCR), 2         ' set only RTS asserted (true)
	FOR k = 1 TO 3: NEXT k        ' small delay
	IF (INP(comadr + MSR) AND 32) = 0 THEN errors = errors + 1  ' get DSR
	OUT (comadr + MCR), 0         ' set RTS deasserted (false)
	FOR k = 1 TO 3: NEXT k        ' small delay
	IF (INP(comadr + MSR) AND 32) = 32 THEN errors = errors + 1 ' get DSR
NEXT i
FOR i = 1 TO 10
	OUT (comadr + MCR), 1         ' set only DTR asserted (true)
	FOR k = 1 TO 3: NEXT k        ' small delay
	IF (INP(comadr + MSR) AND 64) = 0 THEN errors = errors + 1  ' get RI
	OUT (comadr + MCR), 0         ' set DTR deasserted (false)
	FOR k = 1 TO 3: NEXT k        ' small delay
	IF (INP(comadr + MSR) AND 64) = 64 THEN errors = errors + 1 ' get RI
NEXT i

CALL SendSpace25
IF errors = 0 THEN
	PRINT #3, "DIO1 port COM2 Handshake Lines test passed"
ELSE
	CALL Bell
	PRINT #3, "  FAILED !!!  DIO1 port COM2 Handshake Lines test"
END IF

EXIT SUB


readinputs:
  w = (INP(&HF860) AND 32) / 32   ' Read Port1 bit 5  (driven by P3.6)
  x = (INP(&HF860) AND 1)         ' Read Port1 bit 0  (driven by P3.5)
  y = (INP(&HF870) AND 2) / 2     ' Read Port3 bit 1  (driven by P3.0)
  z = (INP(&HF870) AND 8) / 8     ' Read Port3 bit 3  (driven by P3.2)
RETURN

END SUB

SUB DIO2Test

'********************************************************
' Test DIO1 Port     (Loop Back Test)
'
'        OUT            IN                             IN
'
'   LED  (pin 4)  -- IN3  (pin 3)    --Resistor--  TX485EN (pin 12)
'   P1.2 (pin 6)  -- IN2  (pin 5)
'   RTS1 (pin 8)  -- IN1  (pin 7)
'   P3.4 (pin 6)  -- IN4  (pin 1)
'   P1.3 (pin 14) -- DCD2 (pin 10)
'
' Loop Back Cable must connect the above lines
'
'********************************************************

' Initialize Port1 and Port3 I/O pins *****************************

x = (INP(&HF824)) AND &HEF   ' force bit 4 to "0"
OUT &HF824, x                ' set Port3 bit 4 I/O pin
			     ' bit 4 is normally IRQ6

x = (INP(&HF874)) AND &HEF   ' force bit 4 to "0"
OUT &HF874, x                ' set Port3 bit 4 as OUTPUT

x = (INP(&HF872)) OR &H10    ' force bit 4 to "1"
OUT &HF872, x                ' set Port3 bit 4 high OUTPUT

x = (INP(&HF820)) AND &HF3   ' force bit 2 and 3 to "0"
OUT &HF820, x                ' set Port1 bits 2 and 3 to I/O pins

x = (INP(&HF864)) AND &HF3   ' force bit 2 and 3 to "0"
OUT &HF864, x                ' set Port1 bits 2 and 3 to OUTPUTs

x = (INP(&HF862)) AND &HF3   ' force bits 2 and 3 to "0"
OUT &HF862, x                ' set Port1 bits 2 and 3 to zero

OUT (com1adr + MCR), 2       ' set RTS1 asserted (zero)

inputregs.bx = &H100                             ' LED ON
inputregs.ax = &HB010                            ' change LED
CALL interrupt(&H15, inputregs, outputregs)      ' BIOS Interrupt


' **** Start Tests of I/O pins  ***************************************
' t = TX485EN,  v = DCD2,  w = IN1,  x = IN2,  y = IN3,  z = IN4
' at this time:  t = 1,  v = 1,  w = 0,  x = 0,  y = 1,  z = 1

errors = 0                      ' error count
GOSUB readDIO2inputs

IF V + y + z <> 3 THEN errors = errors + 1
IF w + x > 0 THEN errors = errors + 1
IF (INP(rs485adr) AND 1) = 0 THEN           ' if 485 option not present
	IF t = 0 THEN errors = errors + 1   ' then test TX485EN input
END IF

inputregs.bx = &H0                               ' LED OFF
inputregs.ax = &HB010                            ' change LED
CALL interrupt(&H15, inputregs, outputregs)      ' BIOS Interrupt

GOSUB readDIO2inputs             ' y and t = zero now

IF V + z <> 2 THEN errors = errors + 1
IF y + w + x > 0 THEN errors = errors + 1
IF (INP(rs485adr) AND 1) = 0 THEN           ' if 485 option not present
	IF t = 1 THEN errors = errors + 1   ' then test TX485EN input
END IF

OUT (com1adr + MCR), 0            ' set RTS1 deasserted ("1")

GOSUB readDIO2inputs              ' w = 1 now

IF w + V + z <> 3 THEN errors = errors + 1
IF y + x > 0 THEN errors = errors + 1

x = (INP(&HF872)) AND &HEF        ' force bit 4 to "0"
OUT &HF872, x                     ' set Port3 bit 4 low OUTPUT

GOSUB readDIO2inputs              ' z = 0 now

IF w + V <> 2 THEN errors = errors + 1
IF y + x + z > 0 THEN errors = errors + 1

x = (INP(&HF862)) OR &H4          ' force bits 2 to "1"
OUT &HF862, x                     ' set Port1 bit 2 high

GOSUB readDIO2inputs              ' x = 1 now

IF w + V + x <> 3 THEN errors = errors + 1
IF y + z > 0 THEN errors = errors + 1

x = (INP(&HF862)) OR &H8          ' force bits 3 to "1"
OUT &HF862, x                     ' set Port1 bit 3 high

GOSUB readDIO2inputs              ' v = 0 now

IF w + x <> 2 THEN errors = errors + 1
IF V + y + z > 0 THEN errors = errors + 1


CALL SendSpace25
IF errors = 0 THEN
	PRINT #3, "DIO2 port test passed"
ELSE
	CALL Bell
	PRINT #3, "  FAILED !!!  DIO2 port test";
	PRINT #3, errors; "times"
END IF

EXIT SUB

readDIO2inputs:

  t = (INP(&H77) AND 1)                    ' Read TX485EN (driven by LED)
  V = (INP(com2adr + MSR) AND 128) / 128   ' Read DCD2    (driven by P1.3)
  w = (INP(&H77) AND 2) / 2                ' Read IN1     (driven by RTS1)
  x = (INP(&H77) AND 4) / 4                ' Read IN2     (driven by P1.2)
  y = (INP(&H77) AND 8) / 8                ' Read IN3     (driven by LED)
  z = (INP(&H77) AND 16) / 16              ' Read IN4     (driven by P3.4)
RETURN

END SUB

SUB EchoCom1
'********************************************************
' Echo on COM1
'   send 10 char string out on COM1 (RTS asserted for 485)
'   same string should be received on COM1
'       (requires ECHO Test box attached)
'********************************************************

CALL CheckCom1Loop             ' Check for COM1 RS-232 Loop Back cable
IF errors = 0 THEN EXIT SUB    ' Can't do test with it connected

IF Model = 21 THEN OUT &H76, 1           ' enable RS-485
IF Model = 22 THEN OUT &H76, 0           ' disable Automatic RS-485

OUT (com1adr + MCR), &H2                 ' set RTS1 asserted true

GOSUB testecho

IF Model = 21 THEN OUT &H76, 0           ' disable RS-485

CALL SendSpace25

IF J = 0 THEN
	CALL Bell
	PRINT #3, "FAILED RS-485 Test !!!  No characters echoed"
END IF
IF J > 0 AND J < 10 THEN
	CALL Bell
	PRINT #3, "FAILED RS-485 Test !!!  Only "; J; "chars echoed"
END IF

IF J >= 10 THEN
   IF k = 0 THEN
	PRINT #3, "RS-485 Echo Test passed"
   ELSEIF k > 0 THEN
	CALL Bell
	PRINT #3, "FAILED !!!  Echo test ";
	PRINT #3, k;
	PRINT #3, "times"
   END IF
END IF

PRINT #3,

IF Model <> 22 THEN EXIT SUB     ' model 2200 must also test Automatic mode

OUT &H76, 1                      ' enable Automatic RS-485
OUT (com1adr + MCR), &H0         ' set RTS1 deasserted

GOSUB testecho

CALL SendSpace25
IF J = 0 THEN
	CALL Bell
	PRINT #3, "FAILED Automatic RS-485 Test !!!  No chars echoed"
	EXIT SUB
END IF
IF J < 10 THEN
	CALL Bell
	PRINT #3, "FAILED Automatic Test !!!  Only"; J; "chars echoed"
	EXIT SUB
END IF

IF k = 0 THEN
	PRINT #3, "Automatic RS-485 Echo Test passed"
ELSEIF k > 0 THEN
	CALL Bell
	PRINT #3, "FAILED !!!  Automatic RS-485 Echo test ";
	PRINT #3, k;
	PRINT #3, "times"
END IF

EXIT SUB

' *************************** subroutine ***************
testecho:

  FOR i = 1 TO 10            ' Flush COM Rx data
	IF (INP((com1adr + LSR)) AND 1) = 1 THEN x = INP((com1adr + RBR))
  NEXT i

  FOR i = 1 TO 12         ' send 12 char out COM1
	tx = 30 + 7 * i
	CALL SendChar(com1adr, tx)
	FOR k = 1 TO 10      ' delay during sending
	NEXT k               ' BASIC is too slow
  NEXT i
  FOR i = 1 TO 20
  NEXT i                   ' delay to keep RTS asserted

  OUT (com1adr + MCR), &H0                 ' set RTS asserted false

  '****************** now get echoed chars

  k = 0                      ' errors
  J = 0                      ' char count
  FOR i = 1 TO 100           ' timeout after 100 mS ?
  IF (INP((com1adr + LSR)) AND 1) = 1 THEN
	J = J + 1
	x = INP((com1adr + RBR))
	IF x <> 30 + (J * 7) THEN k = k + 1
	i = 1
  END IF
  NEXT i
RETURN

END SUB

SUB EtherInfo
'**************************************************************
'   print ethernet number on the screen
'**************************************************************

inputregs.ax = &HB061                            ' get ethernet info
CALL interrupt(&H15, inputregs, outputregs)      ' BIOS Interrupt

IF INT(outputregs.ax / 256) <> 0 THEN            ' ah
	CALL Bell
	PRINT #3, "  ERROR !!  Ethernet BIOS call returned Ah non-zero"
	PRINT #3, "  Returned value = "; INT(outputregs.ax / 256)
	EXIT SUB
END IF

' x is location 41, y is location 42, z is location 43
' when entering #, x is MSB and z is LSB

x$ = HEX$(INT(outputregs.cx / 256)): x$ = RIGHT$("0" + x$, 2)
y$ = HEX$(INT(outputregs.bx MOD 256)): y$ = RIGHT$("0" + y$, 2)
z$ = HEX$(INT(outputregs.bx / 256)): z$ = RIGHT$("0" + z$, 2)

Eth$ = "00 D0 69   " + x$ + " " + y$ + " " + z$

PRINT #3,
PRINT #3, STRING$(35, " "); "Ethernet # is: "; Eth$

END SUB

SUB EthernetTest

'********************************************************
' Test Ethernet Port
'      must have 10BaseT Test Box connected
'********************************************************

IF rx = 69 OR rx = 101 THEN rx = 69         ' 69 = ASCII "E"

IF rx <> 69 THEN                            ' force console to COM1
	inputregs.ax = &HA100               ' Console redirection call
	inputregs.bx = 1                    ' redirection to COM1
	CALL interrupt(&H15, inputregs, outputregs)      'BIOS Interrupt
END IF

OUT &H2FF, 0                                ' Clear Ping results flag

SHELL "ping /tsepc 192.168.0.200 100"       ' Do 100 packets

inputregs.ax = &HA100                       ' Console redirection call
inputregs.bx = 2                            ' redirect to COM2
CALL interrupt(&H15, inputregs, outputregs) ' BIOS Interrupt

Com2S = INP(&H2FF)     ' Read results flag

IF rx = 69 THEN
	PRINT #3, : PRINT #3,
ELSE
	CALL SendSpace25
END IF

IF Com2S > 127 OR Com2S = 0 THEN
	CALL Bell
	PRINT #3, "  ERROR !! in Ping Ethernet Test"
ELSEIF Com2S < 98 THEN
	CALL Bell
	PRINT #3, "  WARNING !! - Ethernet lost 3 or more packets"
ELSE
	PRINT #3, "Ethernet test passed"
END IF

END SUB

SUB EthNumber
'**********************************************************************
'   This routine allows manual setting of the Ethernet number
'   Based on the unit's serial number
'**********************************************************************

PRINT #3,
PRINT #3, "  This operation can be aborted with the   ESC  key"
PRINT #3,
PRINT #3, "  Enter Serial Number  ( 5 decimal digits) ....  ";

i = 0: rx = 1
WHILE rx <> 27 AND i < 5     ' 27 = ESC key

  CALL GetChar(rx)

  IF rx = 27 THEN
	CALL Bell
	PRINT #3, " ABORT !!"
	EXIT SUB
  END IF


' x is location 41, y is location 42, z is location 43
' when entering #, x is MSB and z is LSB

  IF rx <= 57 AND rx >= 48 THEN     ' decimal digit
	PRINT #3, STRING$(1, rx);
	i = i + 1
	SELECT CASE i
		CASE 1
			PRINT #3, " ";              ' print space
			x = rx - 48                 ' Only digit in 1st byte
		CASE 2
			y = (rx - 48) * 16          ' MS digit of 2nd byte
		CASE 3
			PRINT #3, " ";              ' print space
			y = y + rx - 48             ' LS digit of 2nd byte
		CASE 4
			z = (rx - 48) * 16          ' MS digit of 3rd byte
		CASE 5
			z = z + rx - 48             ' LS digit of 3rd byte
	END SELECT
  END IF

WEND
IF z > 127 THEN z = z - 256     ' fix "overflow problem"
				' bx is 16 bit integer !!
inputregs.bx = z * 256 + y
inputregs.cx = x * 256

inputregs.ax = &HB060                            ' write ethernet #
CALL interrupt(&H15, inputregs, outputregs)      ' BIOS Interrupt
		      
			PRINT #3, : PRINT #3,
			PRINT #3, "  Ethernet number entry completed"
			CALL Continue

END SUB

SUB ExprtMenu
'***********************************************************
'** expert menu...instead of running all standard tests,  **
'** allows one to run a specific test of their choice     **
'***********************************************************

PRINT #3, "*** Choose a test to perform***"
PRINT #3, "        A) Real Time clock test"
PRINT #3, "        B) Test Comm Port 1"
PRINT #3, "        C) Test Comm Port 2"
PRINT #3, "        D) Test 32 pin Socket"
PRINT #3, "            **assumes 32 kb SRAM installed**"
PRINT #3, "        E) DIO1 Loopback Test"
IF Model > 21 THEN
	PRINT #3, "        F) DIO2 Loopback Test"
	PRINT #3, "        G) LCD Test"
	PRINT #3, "        H) Ethernet Test"
END IF

IF (INP(sramadr) AND 1) = 1 THEN
	PRINT #3, "        I) Battery-Backed SRAM Test"
END IF

IF (INP(rs485adr) AND 1) = 1 THEN
	PRINT #3, "        J) Half-Duplex RS-485 Echo Test";
	IF Model = 21 THEN
		PRINT #3, "     (JP7 not installed)"
	ELSE
		PRINT #3, "      (JP7 installed)"
	END IF
	IF Model = 22 THEN
	  PRINT #3, "       K) Full-Duplex RS-485 Test  (JP6 installed)"
	END IF
END IF

IF Model = 28 AND (INP(adcadr) AND 1 = 1) THEN
	PRINT #3, "        L) A/D Converter Test"
END IF

IF Com3Adr <> 0 THEN
	PRINT #3, "        M) COM3 loop back test   (Daughter board)"
END IF
IF Com4Adr <> 0 THEN
	PRINT #3, "        N) COM4 loop back test   (Daughter board)"
END IF

PRINT #3, "       X = Return to Main Menu"


END SUB

SUB ExprtMenuCharDecode (ExitFlag$)
'*******************************************************************
'    get char and decode command for the Expert Menu
'    lifts code made by BLM sub CharDecode
'*******************************************************************

rx = 0

DO
LstChr$ = LCASE$(CHR$(rx))

rx = 0
WHILE rx < 32               ' ignore control char
	CALL GetChar(rx)
WEND

PRINT #3, STRING$(1, rx)

SELECT CASE rx
	CASE 65, 97             'A or a ) real time clock test
		CALL RTCtest
	CASE 66, 98             'B or b ) tests comm port 1
		CALL ComXTest(1, com1adr)
	CASE 67, 99             ' C or c ) tests comm port 2
		CALL ComXTest(2, com2adr)
		
	CASE 68, 100            ' D or d ) test 32 pin socket
		CALL SocketTest
		
	CASE 69, 101            ' E or e ) DIO1 loopback test
		CALL DIO1Test
		
	CASE 70, 102            ' F or f ) DIO2 loopback test
		IF Model > 21 THEN      ' 2100 and below do not support test
			CALL DIO2Test
		
		ELSE
			'do nothing
		END IF
	CASE 71, 103            'G or g ) LCD test
		IF Model > 21 THEN      ' 2100 and below do not support test
			CALL LCDtest
		
		ELSE
			'do nothing
		END IF
	CASE 72, 104            'H or h ) Ethernet Test
		IF Model > 21 THEN      ' 2100 and below do not support test

			FOR i = 1 TO 8: PRINT #3, : NEXT i
			CALL EthernetTest
		
		ELSE
			'do nothing
			PRINT #3, "Test Not Supported on this board"
		END IF
	CASE 73, 105            'I or i ) Battery Backed SRAM test
		IF (INP(sramadr) AND 1) = 1 THEN
			CALL BatteryRAM
		
		ELSE
			'do nothing
			PRINT #3, "Test Not Supported on this board"
		END IF
	CASE 74, 106            'J or j ) Half-duplex RS-485 echo test
		CALL Instruct485
		CALL EchoCom1
		
	CASE 75, 107            'K or k ) Full-duplex RS-485 echo test
		CALL Instruct485
		CALL FD485
		
	CASE 76, 108            'L or l ) a/d converter test
		CALL ADCtest
		
	CASE 77, 109            'M or m ) comm 3 loop back test (D-board)
		CALL ComXTest(3, Com3Adr)
		
	CASE 78, 110            'N or n ) comm 4 loop back test (D-board)
		CALL ComXTest(4, Com4Adr)
		
	CASE 88, 120, 48            ' X or x ) Exit Expert Menu
		ExitFlag$ = "true"
		EXIT SUB
END SELECT

IF LstChr$ = LCASE$(CHR$(rx)) THEN
	'don't print up menu
ELSEIF LstChr$ = CHR$(0) THEN
	'there hasn't been a last key to press
	'so, uhm, don't print menu
ELSE
	''seems funny,but it stores, then puts back, last chr pressed
	''otherwises, the any key to continue pressed becomes last chr,
	''not the option chosen

	CALL ExprtMenu

END IF

LOOP

END SUB

SUB FD485

'********************************************************
'  Full Duplex RS-485 Test
'  This routine requires a loopback cable  TX= to RX+  and  TX- to RX-
'********************************************************

CALL CheckCom1Loop       ' Check for COM1 RS-232 Loop Back cable
IF errors = 0 THEN EXIT SUB

comadr = com1adr

OUT (comadr + MCR), &H2                 ' set RTS asserted true

FOR i = 1 TO 10            ' Flush COM Rx data
	IF (INP((comadr + LSR)) AND 1) = 1 THEN i = INP((comadr + RBR))
NEXT i

CALL SendChar(comadr, 92)
CALL SendChar(comadr, 45)

t = 0: z = 0
FOR i = 1 TO 50
	IF (INP(comadr + LSR) AND 1) THEN
		t = INP(comadr + RBR)
		EXIT FOR
	END IF
NEXT i                                  ' got 1st char
FOR i = 1 TO 50
	x = INP(comadr + LSR) AND 1
	IF x = 1 THEN
		z = INP(comadr + RBR)
		EXIT FOR
	END IF
NEXT i                     ' got 2nd char

CALL SendSpace25

IF t = 92 AND z = 45 THEN
	PRINT #3, "RS-485 Full Duplex test passed"
ELSE
	CALL Bell
	PRINT #3, "  FAILED !!!  RS-485 Full Duplex test"
END IF


END SUB

SUB GetChar (rx)

'  This waits for character received in COM2
'  returns character in rx as a value 0-255
'  while waiting it blinks the LED
'  returns with LED off
'********************************************************

rx = 300
WHILE rx > 260

  inputregs.bx = &H100                             ' LED ON
  inputregs.ax = &HB010                            ' change LED
  CALL interrupt(&H15, inputregs, outputregs)      ' BIOS Interrupt

  FOR i = 1 TO 750
	IF i = 150 THEN
		inputregs.bx = &H0                          ' LED OFF
		inputregs.ax = &HB010                       ' change LED
		CALL interrupt(&H15, inputregs, outputregs) ' BIOS Interrupt
	END IF
	x = INP(com2lsr) AND 1
	IF x = 1 THEN
		rx = INP(com2data)
		EXIT FOR
	END IF
  NEXT i

  inputregs.bx = &H0                          ' LED OFF
  inputregs.ax = &HB010                       ' change LED
  CALL interrupt(&H15, inputregs, outputregs) ' BIOS Interrupt

WEND

END SUB

SUB Instruct485

'********************************************************
' List Instructions for RS-485 Tests
'********************************************************

FOR i = 1 TO 6: PRINT #3, : NEXT i

IF (INP(rs485adr) AND 1) = 1 THEN
	PRINT #3,
	PRINT #3, " For RS-485 Tests -- COM1 Loop Back cable must be removed"
	PRINT #3,
	IF Model = 22 THEN
	   PRINT #3, "     ";
	   IF rx = 72 OR rx = 104 THEN         '  H or h
	       PRINT #3, "Half-Duplex test - JP7 installed, JP6 removed"
	       PRINT #3, "               ";
	       PRINT #3, "         and Echo Test Box connected"
	   ELSE
	       PRINT #3, "Full-Duplex test - JP6 installed, JP7 removed"
	       PRINT #3, "               ";
	       PRINT #3, "         and 485 Loop Back Plug connected"
	   END IF
	ELSE
	   PRINT #3, "                  -- and Echo Test Box connected"
	END IF
	IF Model = 21 THEN
	  PRINT #3,
	  PRINT #3, " JP7 must be removed for this Test"
	END IF

ELSE
	PRINT #3, "MACH does not have this option enabled"
END IF

PRINT #3, : PRINT #3,

END SUB

SUB Instructions

'********************************************************
' List General Instructions
'********************************************************

FOR i = 1 TO 12: PRINT #3, : NEXT i

IF Model = 21 THEN
	PRINT #3, " Model 2100";
ELSEIF Model = 22 THEN
	PRINT #3, " Model 2200";
ELSE
	PRINT #3, " Model 2800";
END IF

PRINT #3, " Instructions  --";
PRINT #3, " LED D2 should be blinking"
CALL SendSpace25
IF Model = 21 THEN PRINT #3, " -- JP2 and JP7 should be installed"
IF Model = 22 THEN PRINT #3, " -- JP2, JP4, JP5, and JP7 should  be installed"
IF Model = 28 THEN PRINT #3, " -- JP2 thru JP9 (8 total) should  be installed"

PRINT #3,
PRINT #3, " Always run the -R- test"
PRINT #3, " Always run the -O- test"

IF (INP(rs485adr) AND 1) = 1 THEN
	PRINT #3,
	PRINT #3, " This unit requires -H- ";
	IF Model = 22 THEN PRINT #3, "and -F- ";
	PRINT #3, "Test"
END IF

IF Model = 28 AND (INP(adcadr) AND 1) = 1 THEN
	PRINT #3,
	PRINT #3, " This unit requires -A- Test"
END IF

IF (INP(sramadr) AND 1) = 1 THEN
	PRINT #3,
	PRINT #3, " To do a Battery-backed SRAM Test:"
	PRINT #3, "      - Do a -R- test first  (SRAM is Initialized)"
	PRINT #3, "      - Then turn Power OFF  (leave off a few seconds)"
	PRINT #3, "      - Turn Power ON"
	PRINT #3, "      - Run -B- test    (before running R test)"
END IF

IF Model > 21 THEN
	PRINT #3,
	PRINT #3, " Verify Ethernet serial number is correct"
	PRINT #3, "   - LED D4 should be ON when Ethernet cable connected"
	PRINT #3, "   - LED D3 should blink ON during -E- test"
END IF

PRINT #3,
PRINT #3, " Verify Date and Time is correct"

CALL Continue

END SUB

SUB LCDtest
'********************************************************
' Test LCD display
'********************************************************

IF lcdflag = 0 THEN
	OUT lcdcom, &H30           ' set to 8-bit bus
	FOR k = 1 TO 50: NEXT k    ' must wait 5 mS
	OUT lcdcom, &H30           ' set to 8-bit bus
	FOR k = 1 TO 4: NEXT k     ' must wait .1 mS
	OUT lcdcom, &H30           ' set to 8-bit bus
	FOR k = 1 TO 4: NEXT k     ' must wait .1 mS
	lcdflag = 1
END IF

' check for LCD presence"

CALL SendSpace25

IF INP(lcdcom) > 127 THEN
	CALL Bell
	PRINT #3, "  ERROR *******  No LCD present  "
	lcdflag = 0
	EXIT SUB
END IF

PRINT #3, "LCD presence detected"

IF lcdflag = 1 THEN
   OUT lcdcom, &H38          ' set to 2 lines, 5 x 7 dot per character
   WAIT lcdcom, 128, 128
   OUT lcdcom, &H8            ' set display off, cursor off, blink off
   WAIT lcdcom, 128, 128
   OUT lcdcom, &H1            ' clear display and set DD RAM address to zero
   WAIT lcdcom, 128, 128
   OUT lcdcom, &H6            ' cursor increments, display shift off ?
   WAIT lcdcom, 128, 128

   OUT lcdcom, &H38           ' ?     Maris
   WAIT lcdcom, 128, 128
   OUT lcdcom, &HC            ' ????
   WAIT lcdcom, 128, 128
   OUT lcdcom, &H6            ' cursor increments, display shift off ?
   WAIT lcdcom, 128, 128
   OUT lcdcom, &H1            ' clear display and set pointer to zero
   WAIT lcdcom, 128, 128

   OUT lcddat, &H4C            ' ASCII L
   WAIT lcdcom, 128, 128
   OUT lcddat, &H43            ' ASCII C
   WAIT lcdcom, 128, 128
   OUT lcddat, &H44            ' ASCII D
   WAIT lcdcom, 128, 128

   OUT lcddat, &H20            ' ASCII space
   WAIT lcdcom, 128, 128
   OUT lcddat, &H77            ' ASCII w
   WAIT lcdcom, 128, 128
   OUT lcddat, &H6F            ' ASCII o
   WAIT lcdcom, 128, 128
   OUT lcddat, &H72            ' ASCII r
   WAIT lcdcom, 128, 128
   OUT lcddat, &H6B            ' ASCII k
   WAIT lcdcom, 128, 128
   OUT lcddat, &H73            ' ASCII s
   WAIT lcdcom, 128, 128

   OUT lcddat, &H20            ' ASCII space
   WAIT lcdcom, 128, 128
   OUT lcddat, &H4F            ' ASCII O
   WAIT lcdcom, 128, 128
   OUT lcddat, &H4B            ' ASCII K
   WAIT lcdcom, 128, 128
   lcdflag = 2
ELSE
   OUT lcddat, &H2A            ' asteric
END IF

END SUB

SUB Menu
'****************************************************************
'    Print Menu
'****************************************************************

PRINT #3, : PRINT #3,
PRINT #3, "Command"; STRING$(28, " "); DATE$; STRING$(16, " "); TIME$

IF Model > 21 THEN CALL EtherInfo      ' print Ethernet #

PRINT #3,
PRINT #3, "       - R = Run Standard Tests"
PRINT #3, "       - S = Choose Single Test to Perform"

IF (INP(sramadr) AND 1) = 1 THEN
	PRINT #3, "       - B = Battery-Backed SRAM Test"
END IF
IF (INP(rs485adr) AND 1) = 1 THEN
	PRINT #3, "       - H = Half-Duplex RS-485 Echo Test";
	IF Model = 21 THEN
		PRINT #3, "  (JP7 not installed)"
	ELSE
		PRINT #3, "  (JP7 installed)"
	END IF
	IF Model = 22 THEN
	  PRINT #3, "       - F = Full-Duplex RS-485 Test  (JP6 installed)"
	END IF
END IF
IF Model > 21 THEN
	PRINT #3, "       - E = Ethernet Test"
	PRINT #3, "       - N = Enter Ethernet Number"
END IF
PRINT #3, "       - I = Instructions"
PRINT #3, "       - O = Options Present"
IF Model = 28 AND (INP(adcadr) AND 1 = 1) THEN
	PRINT #3, "       - A = A/D Converter Test"
END IF
IF Com3Adr <> 0 THEN
	PRINT #3, "       - 3 = COM3 loop back test   (Daughter board)"
END IF
IF Com4Adr <> 0 THEN
	PRINT #3, "       - 4 = COM4 loop back test   (Daughter board)"
END IF

PRINT #3, "       - X = exit"


END SUB

SUB Options
'********************************************************
' List Options
'********************************************************

PRINT #3,

IF Model = 21 THEN
	PRINT #3, "  Model 2100";
ELSEIF Model = 22 THEN
	PRINT #3, "  Model 2200";
ELSE
	PRINT #3, "  Model 2800";
END IF

PRINT #3, " has these options present:"
PRINT #3,

s = INP(sramadr) AND 1
IF s = 1 THEN
	IF sram = 32 THEN PRINT #3, "      - 32K SRAM option"
	IF sram = 128 THEN PRINT #3, "      - 128K SRAM option"
END IF

R = INP(rs485adr) AND 1
IF R = 1 THEN PRINT #3, "      - RS-485 option"

a = 0
IF Model = 28 THEN            ' model 2800 only
	a = INP(adcadr) AND 1
	IF a = 1 THEN PRINT #3, "      - A/D converter option"
END IF
IF s = 0 AND R = 0 AND a = 0 THEN
	PRINT #3, "  There are NO options present"
END IF


CALL Continue


END SUB

SUB partest
'********************************************************
'    Test parallel port and OPTO-Isolated Inputs
'********************************************************
' assumes each data bit OUT is connected to data bit IN
' also assumes OPTO-isolated inputs are connected
' if model <> 23 then
'               opto tests skipped
'               and strobe tested differently
'

IF Model = 23 THEN         ' model 2325
	y = 0
	FOR i = 0 TO 255
		OUT pardata, i
		IF i <> INP(parstatus) THEN y = y + 1
	NEXT i
ELSE                        ' model 1325, 2350
	OUT p3ltc, 0                           ' set strobe = 1
	y = 0
	FOR i = 0 TO 127
		OUT pardata, i + 128
		IF i <> INP(parstatus) THEN y = y + 1
	NEXT i
	FOR i = 128 TO 255
		OUT pardata, i - 128
		IF i <> INP(parstatus) THEN y = y + 1
	NEXT i
 
	OUT pardata, 128                       ' set data 7 = 1
	OUT p3ltc, 64                          ' set strobe = 0
	IF INP(parstatus) <> 128 THEN y = y + 1
	OUT p3ltc, 0                           ' set strobe = 1
END IF

CALL SendSpace25
IF y = 0 THEN
	PRINT #3, "Parallel port passed pattern test"
ELSEIF y <> 0 THEN
	CALL Bell: CALL Bell
	PRINT #3, "FAILED "
	PRINT #3, y;
	PRINT #3, " times !!!    parallel test"
END IF

END SUB

SUB RTCtest
'********************************************************
'    Test Real-time Clock
'********************************************************

y = 0                                       ' error count
OUT rtcadd, 125: rtcsave = INP(rtcdata)     ' save old data
OUT rtcadd, 126: rtcsav2 = INP(rtcdata)     ' save old data

FOR i = 0 TO 255
	OUT rtcadd, 125
	OUT rtcdata, i
	OUT rtcadd, 126
	OUT rtcdata, 12
	OUT rtcadd, 125
	IF i <> INP(rtcdata) THEN y = y + 1
NEXT i
OUT rtcadd, 125: OUT rtcdata, rtcsave       ' restore original data
OUT rtcadd, 126: OUT rtcdata, rtcsav2       ' restore original data

CALL SendSpace25
IF y = 0 THEN
	PRINT #3, "Real-Time Clock passed pattern test"
ELSEIF y <> 0 THEN
	CALL Bell
	PRINT #3, "  FAILED !!!    Real-Time Clock test"
END IF

END SUB

SUB RunStandard
'*******************************************************************
'  This routine runs a standard set of tests
'*******************************************************************

FOR i = 1 TO 8: PRINT #3, : NEXT i

CALL RTCtest
CALL ComXTest(1, com1adr)
CALL ComXTest(2, com2adr)
CALL SocketTest
CALL DIO1Test

IF Model > 21 THEN             ' check if model 2200 or 2800
	CALL LCDtest
	CALL DIO2Test
	CALL EthernetTest
END IF

CALL Continue

END SUB

SUB SendChar (comadr, TxD)
'********************************************************
' sends char in TxD out serial port with base adr ComAdr
' waits till previous char sent before sending
'********************************************************
TbrFull:
	x = INP(comadr + LSR) AND 32    ' is holding reg empty ?
	IF x = 0 THEN GOTO TbrFull      ' wait till ready
	OUT (comadr + TBR), TxD
END SUB

SUB SendSpace25
'********************************************************
' send 25 spaces out COM2
'********************************************************
	FOR i = 1 TO 25
		CALL SendChar(com2adr, &H20)
	NEXT i
END SUB

SUB SocketTest
'********************************************************
' test 32 pin socket,  assumes 32 Kbyte SRAM installed
'********************************************************

IF Model = 28 THEN
	DEF SEG = &HD000           ' 32 Kbyte window to SRAM
ELSEIF sram = 128 THEN
	DEF SEG = &HC000           ' 64K range  (2 pages)
	OUT srampage, 0            ' set to page 0
ELSE
	DEF SEG = &HC800           ' 32 Kbyte window to SRAM
END IF

CALL SendSpace25

' check for Disk-on-Chip
' and Abort if present
IF PEEK(24576) = &H55 AND PEEK(24577) = &HAA THEN
	CALL Bell
	PRINT #3, "  Socket Test ABORTED  --  DOC installed"
	EXIT SUB
END IF

IF (INP(sramadr) AND 1) = 1 THEN     ' check for SRAM option
	z = 0
	y = 0
	FOR i = 1 TO 17000 STEP 15
		z = (z + 37) MOD 256
		POKE i, z
	NEXT i
	z = 0
	FOR i = 1 TO 17000 STEP 15
		z = (z + 37) MOD 256
		IF PEEK(i) <> z THEN y = y + 1
	NEXT i
	IF y = 0 THEN
		PRINT #3, "32K SRAM Initialization test passed"
	ELSEIF y <> 0 THEN
		CALL Bell
		PRINT #3, "  FAILED !!!  32K SRAM Initialization test"
	END IF
	IF sram = 128 THEN
		OUT srampage, 1         ' set to Page 1
		CALL SendSpace25
		z = 0
		y = 0
		FOR i = 1 TO 17000 STEP 15
			z = (z + 31) MOD 256
			POKE i, z
		NEXT i
		z = 0
		FOR i = 1 TO 17000 STEP 15
			z = (z + 31) MOD 256
			IF PEEK(i) <> z THEN y = y + 1
		NEXT i
	       
		OUT srampage, 0         ' set back to Page 0
		z = 0
		FOR i = 1 TO 17 STEP 15
			z = (z + 37) MOD 256
			IF PEEK(i) <> z THEN y = y + 1
		NEXT i
	       
		IF y = 0 THEN
			PRINT #3, "128K SRAM Initialization test passed"
		ELSEIF y <> 0 THEN
		     CALL Bell
		     PRINT #3, "  FAILED !!!  128K SRAM Initialization test"
		END IF
	END IF
ELSE
	z = 0
	y = 0
	FOR i = 26000 TO 31000 STEP 15
		z = (z + 37) MOD 256
		POKE i, z
	NEXT i
	z = 0
	FOR i = 26000 TO 31000 STEP 15
		z = (z + 37) MOD 256
		IF PEEK(i) <> z THEN y = y + 1
	NEXT i
	IF y = 0 THEN
		PRINT #3, "Socket test passed"
	ELSEIF y <> 0 THEN
		CALL Bell
		PRINT #3, "  FAILED !!!  Socket test"
	END IF
END IF

END SUB

SUB StartupScreen

'********************************************************
' List Start up Screen
'********************************************************

FOR i = 1 TO 10: PRINT #3, : NEXT i

PRINT #3, "  Diagnostic program for Technologic Systems Embedded PC"
PRINT #3, "         Diagnostic Program Version " + VERSION
PRINT #3,
PRINT #3,
PRINT #3, "  This program is for models TS-2100, TS-2200, and TS-2800"
PRINT #3,
PRINT #3, "  Special test cables must be attached for these tests"
PRINT #3,

inputregs.ax = &HB000                            ' get model info
CALL interrupt(&H15, inputregs, outputregs)      'BIOS Interrupt

' Check for "TS" in dx
IF (outputregs.dx <> &H5453) THEN
	CALL Bell
	PRINT #3, " Error  ******  Wrong BIOS !!!  ******"
	PRINT #3, "       This is not a TS BIOS"
END IF

IF INT(outputregs.ax / 256) <> 0 THEN
	PRINT "This program is for use only with Technologic Systems products"
	END                 ' Exit the program
END IF
       
Model = outputregs.cx MOD 256            ' CL    (model)
spnumber = outputregs.ax MOD 256         ' save SP Number

IF spnumber = 255 THEN                   ' then it's a dev. version
	CALL Bell:  PRINT #3,
	PRINT #3, "!!! BETA TEST BIOS VERSION - NOT FOR PUBLIC RELEASE !!!"
	PRINT #3, : PRINT #3,
END IF

PRINT #3,
PRINT #3, "  This board is a ";
IF Model = 21 THEN PRINT #3, "Model TS-2100";
IF Model = 22 THEN PRINT #3, "Model TS-2200";
IF Model = 28 THEN PRINT #3, "Model TS-2800";
PRINT #3, " with BIOS Version ";

x$ = STR$(INT(outputregs.bx / 256))     ' BH  (msb Version)
y = INT(outputregs.bx MOD 256)          ' BL  (lsb Version)

y$ = STR$(y)
IF y < 10 THEN
	y$ = "0" + RIGHT$(y$, 1)
ELSE
	y$ = RIGHT$(y$, 2)
END IF
PRINT #3, x$ + "." + y$
PRINT #3,

sp$ = STR$(spnumber)
IF spnumber > 0 THEN
	PRINT #3, "  The BIOS in this unit has a Special Product number:";
	PRINT #3, "  SP#" + sp$
END IF

IF Model = 22 AND (INP(sramsiz) AND 1) = 1 THEN
	sram = 128         ' For 64K range starting at C0000h
	OUT &HF420, &H80         ' F420 = 0580h
	OUT &HF421, &H5
	OUT &HF422, &HC          ' F422 = 000Ch
	OUT &HF423, &H0
	OUT &HF424, &H1          ' F424 = FC01h
	OUT &HF425, &HFC
	OUT &HF426, &H0          ' F426 = 0000h
	OUT &HF427, &H0
	PRINT #3, : PRINT #3, "  128K SRAM option present"
ELSE
	sram = 32
END IF

CALL Continue

FOR i = 1 TO 3: PRINT #3, : NEXT i

END SUB

